@@ -6,7 +6,6 @@ import anndata
 import numpy as np
 import pandas as pd
 
-from scvi.data import setup_anndata
 from scvi.data._built_in_data._download import _download
 
 logger = logging.getLogger(__name__)
@@ -15,7 +14,6 @@ logger = logging.getLogger(__name__)
 def _load_seqfishplus(
     save_path: str = "data/",
     tissue_region: str = "subventricular cortex",
-    run_setup_anndata: bool = True,
 ) -> anndata.AnnData:
 
     if tissue_region == "subventricular cortex":
@@ -40,8 +38,6 @@ def _load_seqfishplus(
     adata.obs["batch"] = np.zeros(adata.shape[0], dtype=np.int64)
     adata.obs["labels"] = np.zeros(adata.shape[0], dtype=np.int64)
 
-    if run_setup_anndata:
-        setup_anndata(adata, batch_key="batch", labels_key="labels")
     return adata
 
 
@@ -70,9 +66,7 @@ def _load_seqfishplus_data(
     return adata
 
 
-def _load_seqfish(
-    save_path: str = "data/", run_setup_anndata: bool = True
-) -> anndata.AnnData:
+def _load_seqfish(save_path: str = "data/") -> anndata.AnnData:
     save_path = os.path.abspath(save_path)
     url = "https://www.cell.com/cms/attachment/2080562255/2072099886/mmc6.xlsx"
     save_fn = "SeqFISH.xlsx"
@@ -80,8 +74,6 @@ def _load_seqfish(
     adata = _load_seqfish_data(os.path.join(save_path, save_fn))
     adata.obs["batch"] = np.zeros(adata.shape[0], dtype=np.int64)
     adata.obs["labels"] = np.zeros(adata.shape[0], dtype=np.int64)
-    if run_setup_anndata:
-        setup_anndata(adata, batch_key="batch", labels_key="labels")
     return adata
 
 
