@@ -4,8 +4,6 @@ import numpy as np
 import pandas as pd
 from anndata import AnnData
 
-from scvi.data import setup_anndata
-
 logger = logging.getLogger(__name__)
 
 
@@ -15,7 +13,6 @@ def _generate_synthetic(
     n_proteins: int = 100,
     n_batches: int = 2,
     n_labels: int = 3,
-    run_setup_anndata: bool = True,
 ) -> AnnData:
 
     data = np.random.negative_binomial(5, 0.3, size=(batch_size * n_batches, n_genes))
@@ -36,13 +33,5 @@ def _generate_synthetic(
     p_data = np.random.negative_binomial(5, 0.3, size=(adata.shape[0], n_proteins))
     adata.obsm["protein_expression"] = p_data
     adata.uns["protein_names"] = np.arange(n_proteins).astype(str)
-    if run_setup_anndata:
-        setup_anndata(
-            adata,
-            batch_key="batch",
-            labels_key="labels",
-            protein_expression_obsm_key="protein_expression",
-            protein_names_uns_key="protein_names",
-        )
 
     return adata
