@@ -8,16 +8,17 @@ import numpy as np
 import pandas as pd
 import torch
 from anndata import AnnData
-from pytorch_lightning.callbacks.early_stopping import EarlyStopping
 
-from scvi import _CONSTANTS
-from scvi.data import get_from_registry, setup_anndata, transfer_anndata_setup
+from scvi import REGISTRY_KEYS
+from scvi.data import AnnDataManager
+from scvi.data.fields import CategoricalObsField, LayerField
 from scvi.dataloaders import DataSplitter
 from scvi.model import SCVI
 from scvi.model.base import BaseModelClass
 from scvi.module import Classifier
 from scvi.module.base import auto_move_data
-from scvi.train import ClassifierTrainingPlan, TrainRunner
+from scvi.train import ClassifierTrainingPlan, LoudEarlyStopping, TrainRunner
+from scvi.utils import setup_anndata_dsp
 
 logger = logging.getLogger(__name__)
 
@@ -35,7 +36,7 @@ class SOLO(BaseModelClass):
     Parameters
     ----------
     adata
-        AnnData object that has been registered via :func:`~scvi.data.setup_anndata`.
+        AnnData object that has been registered via :meth:`~scvi.model.SCVI.setup_anndata`.
         Object should contain latent representation of real cells and doublets as `adata.X`.
         Object should also be registered, using `.X` and `labels_key="_solo_doub_sim"`.
     **classifier_kwargs
@@ -46,7 +47,7 @@ class SOLO(BaseModelClass):
     In the case of scVI trained with multiple batches:
 
     >>> adata = anndata.read_h5ad(path_to_anndata)
-    >>> scvi.data.setup_anndata(adata, batch_key="batch")
+    >>> scvi.model.SCVI.setup_anndata(adata, batch_key="batch")
     >>> vae = scvi.model.SCVI(adata)
     >>> vae.train()
     >>> solo_batch_1 = scvi.external.SOLO.from_scvi_model(vae, restrict_to_batch="batch 1")
@@ -56,7 +57,7 @@ class SOLO(BaseModelClass):
     Otherwise:
 
     >>> adata = anndata.read_h5ad(path_to_anndata)
-    >>> scvi.data.setup_anndata(adata)
+    >>> scvi.model.SCVI.setup_anndata(adata)
     >>> vae = scvi.model.SCVI(adata)
     >>> vae.train()
     >>> solo = scvi.external.SOLO.from_scvi_model(vae)
@@ -82,7 +83,7 @@ class SOLO(BaseModelClass):
         super().__init__(adata)
 
         self.module = Classifier(
-            n_input=self.summary_stats["n_vars"],
+            n_input=self.summary_stats.n_vars,
             n_labels=2,
             logits=True,
             **classifier_kwargs,
@@ -126,15 +127,20 @@ class SOLO(BaseModelClass):
         SOLO model
         """
         _validate_scvi_model(scvi_model, restrict_to_batch=restrict_to_batch)
-        orig_adata = scvi_model.adata
-        orig_batch_key = scvi_model.scvi_setup_dict_["categorical_mappings"][
-            "_scvi_batch"
-        ]["original_key"]
+        orig_adata_manager = scvi_model.adata_manager
+        orig_batch_key = orig_adata_manager.get_state_registry(
+            REGISTRY_KEYS.BATCH_KEY
+        ).original_key
+        orig_labels_key = orig_adata_manager.get_state_registry(
+            REGISTRY_KEYS.LABELS_KEY
+        ).original_key
 
         if adata is not None:
-            transfer_anndata_setup(orig_adata, adata)
+            adata_manager = orig_adata_manager.transfer_fields(adata)
+            cls.register_manager(adata_manager)
         else:
-            adata = orig_adata
+            adata_manager = orig_adata_manager
+        adata = adata_manager.adata
 
         if restrict_to_batch is not None:
             batch_mask = adata.obs[orig_batch_key] == restrict_to_batch
@@ -153,7 +159,7 @@ class SOLO(BaseModelClass):
 
         # anndata with only generated doublets
         doublet_adata = cls.create_doublets(
-            adata, indices=batch_indices, doublet_ratio=doublet_ratio
+            adata_manager, indices=batch_indices, doublet_ratio=doublet_ratio
         )
         # if scvi wasn't trained with batch correction having the
         # zeros here does nothing.
@@ -161,6 +167,12 @@ class SOLO(BaseModelClass):
             restrict_to_batch if restrict_to_batch is not None else 0
         )
 
+        # Create dummy labels column set to first label in adata (does not affect inference).
+        dummy_label = orig_adata_manager.get_state_registry(
+            REGISTRY_KEYS.LABELS_KEY
+        ).categorical_mapping[0]
+        doublet_adata.obs[orig_labels_key] = dummy_label
+
         # if model is using observed lib size, needs to get lib sample
         # which is just observed lib size on log scale
         give_mean_lib = not scvi_model.module.use_observed_lib_size
@@ -182,7 +194,6 @@ class SOLO(BaseModelClass):
         logger.info("Creating doublets, preparing SOLO model.")
         f = io.StringIO()
         with redirect_stdout(f):
-            setup_anndata(doublet_adata, batch_key=orig_batch_key)
             doublet_latent_rep = scvi_model.get_latent_representation(doublet_adata)
             doublet_lib_size = scvi_model.get_latent_library_size(
                 doublet_adata, give_mean=give_mean_lib
@@ -193,12 +204,13 @@ class SOLO(BaseModelClass):
             doublet_adata.obs[LABELS_KEY] = "doublet"
 
             full_adata = latent_adata.concatenate(doublet_adata)
-            setup_anndata(full_adata, labels_key=LABELS_KEY)
+            cls.setup_anndata(full_adata, labels_key=LABELS_KEY)
         return cls(full_adata, **classifier_kwargs)
 
-    @staticmethod
+    @classmethod
     def create_doublets(
-        adata: AnnData,
+        cls,
+        adata_manager: AnnDataManager,
         doublet_ratio: int,
         indices: Optional[Sequence[int]] = None,
         seed: int = 1,
@@ -208,7 +220,7 @@ class SOLO(BaseModelClass):
         Parameters
         ----------
         adata
-            AnnData object setup with :func:`~scvi.data.setup_anndata`.
+            AnnData object setup with setup_anndata.
         doublet_ratio
             Ratio of generated doublets to produce relative to number of
             cells in adata or length of indices, if not `None`.
@@ -217,11 +229,12 @@ class SOLO(BaseModelClass):
         seed
             Seed for reproducibility
         """
+        adata = adata_manager.adata
         n_obs = adata.n_obs if indices is None else len(indices)
         num_doublets = doublet_ratio * n_obs
 
         # counts can be in many locations, this uses where it was registered in setup
-        x = get_from_registry(adata, _CONSTANTS.X_KEY)
+        x = adata_manager.get_from_registry(REGISTRY_KEYS.X_KEY)
         if indices is not None:
             x = x[indices]
 
@@ -236,11 +249,7 @@ class SOLO(BaseModelClass):
         ]
 
         # if adata setup with a layer, need to add layer to doublets adata
-        data_registry = adata.uns["_scvi"]["data_registry"]
-        x_loc = data_registry[_CONSTANTS.X_KEY]["attr_name"]
-        layer = (
-            data_registry[_CONSTANTS.X_KEY]["attr_key"] if x_loc == "layers" else None
-        )
+        layer = adata_manager.data_registry[REGISTRY_KEYS.X_KEY].attr_key
         if layer is not None:
             doublets_ad.layers[layer] = doublets
 
@@ -301,7 +310,7 @@ class SOLO(BaseModelClass):
 
         if early_stopping:
             early_stopping_callback = [
-                EarlyStopping(
+                LoudEarlyStopping(
                     monitor="validation_loss",
                     min_delta=early_stopping_min_delta,
                     patience=early_stopping_patience,
@@ -321,7 +330,7 @@ class SOLO(BaseModelClass):
         plan_kwargs = plan_kwargs if isinstance(plan_kwargs, dict) else dict()
 
         data_splitter = DataSplitter(
-            self.adata,
+            self.adata_manager,
             train_size=train_size,
             validation_size=validation_size,
             batch_size=batch_size,
@@ -351,6 +360,7 @@ class SOLO(BaseModelClass):
             Return probabilities instead of class label
         include_simulated_doublets
             Return probabilities for simulated doublets as well.
+
         Returns
         -------
         DataFrame with prediction, index corresponding to cell barcode.
@@ -367,7 +377,7 @@ class SOLO(BaseModelClass):
 
         y_pred = []
         for _, tensors in enumerate(scdl):
-            x = tensors[_CONSTANTS.X_KEY]
+            x = tensors[REGISTRY_KEYS.X_KEY]
             pred = auto_forward(self.module, x)
             y_pred.append(pred.cpu())
 
@@ -378,9 +388,9 @@ class SOLO(BaseModelClass):
 
         preds = y_pred[mask]
 
-        cols = self.adata.uns["_scvi"]["categorical_mappings"]["_scvi_labels"][
-            "mapping"
-        ]
+        cols = self.adata_manager.get_state_registry(
+            REGISTRY_KEYS.LABELS_KEY
+        ).categorical_mapping
         preds_df = pd.DataFrame(preds, columns=cols, index=self.adata.obs_names[mask])
 
         if not soft:
@@ -388,9 +398,37 @@ class SOLO(BaseModelClass):
 
         return preds_df
 
+    @classmethod
+    @setup_anndata_dsp.dedent
+    def setup_anndata(
+        cls,
+        adata: AnnData,
+        labels_key: Optional[str] = None,
+        layer: Optional[str] = None,
+        **kwargs,
+    ):
+        """
+        %(summary)s.
+
+        Parameters
+        ----------
+        %(param_labels_key)s
+        %(param_layer)s
+        """
+        setup_method_args = cls._get_setup_method_args(**locals())
+        anndata_fields = [
+            LayerField(REGISTRY_KEYS.X_KEY, layer, is_count_data=True),
+            CategoricalObsField(REGISTRY_KEYS.LABELS_KEY, labels_key),
+        ]
+        adata_manager = AnnDataManager(
+            fields=anndata_fields, setup_method_args=setup_method_args
+        )
+        adata_manager.register_fields(adata, **kwargs)
+        cls.register_manager(adata_manager)
+
 
 def _validate_scvi_model(scvi_model: SCVI, restrict_to_batch: str):
-    if scvi_model.summary_stats["n_batch"] > 1 and restrict_to_batch is None:
+    if scvi_model.summary_stats.n_batch > 1 and restrict_to_batch is None:
         warnings.warn(
             "Solo should only be trained on one lane of data using `restrict_to_batch`. Performance may suffer.",
             UserWarning,
