@@ -6,7 +6,9 @@ from typing import Optional, Union
 import torch
 from anndata import AnnData
 
-from scvi.data import transfer_anndata_setup
+from scvi.data import _constants
+from scvi.data._compat import manager_from_setup_dict
+from scvi.data._constants import _MODEL_NAME_KEY, _SETUP_ARGS_KEY
 from scvi.model._utils import parse_use_gpu_arg
 from scvi.nn import FCLayers
 
@@ -35,13 +37,13 @@ class ArchesMixin:
         freeze_classifier: bool = True,
     ):
         """
-        Online update of a reference model with scArches algorithm [Lotfollahi20]_.
+        Online update of a reference model with scArches algorithm [Lotfollahi21]_.
 
         Parameters
         ----------
         adata
             AnnData organized in the same way as data used to train model.
-            It is not necessary to run :func:`~scvi.data.setup_anndata`,
+            It is not necessary to run setup_anndata,
             as AnnData is validated against the saved `scvi` setup dictionary.
         reference_model
             Either an already instantiated model of the same class, or a path to
@@ -69,19 +71,12 @@ class ArchesMixin:
         """
         use_gpu, device = parse_use_gpu_arg(use_gpu)
         if isinstance(reference_model, str):
-            (
-                scvi_setup_dict,
-                attr_dict,
-                var_names,
-                load_state_dict,
-                _,
-            ) = _load_saved_files(
+            attr_dict, var_names, load_state_dict, _ = _load_saved_files(
                 reference_model, load_adata=False, map_location=device
             )
         else:
             attr_dict = reference_model._get_user_attributes()
             attr_dict = {a[0]: a[1] for a in attr_dict if a[0][-1] == "_"}
-            scvi_setup_dict = attr_dict.pop("scvi_setup_dict_")
             var_names = reference_model.adata.var_names
             load_state_dict = deepcopy(reference_model.module.state_dict())
 
@@ -90,20 +85,45 @@ class ArchesMixin:
             adata._inplace_subset_var(var_names)
         _validate_var_names(adata, var_names)
 
-        version_split = scvi_setup_dict["scvi_version"].split(".")
+        if "scvi_setup_dict_" in attr_dict:
+            scvi_setup_dict = attr_dict.pop("scvi_setup_dict_")
+            cls.register_manager(
+                manager_from_setup_dict(
+                    cls, adata, scvi_setup_dict, extend_categories=True
+                )
+            )
+        else:
+            registry = attr_dict.pop("registry_")
+            if (
+                _MODEL_NAME_KEY in registry
+                and registry[_MODEL_NAME_KEY] != cls.__name__
+            ):
+                raise ValueError(
+                    "It appears you are loading a model from a different class."
+                )
+
+            if _SETUP_ARGS_KEY not in registry:
+                raise ValueError(
+                    "Saved model does not contain original setup inputs. "
+                    "Cannot load the original setup."
+                )
+
+            cls.setup_anndata(
+                adata,
+                source_registry=registry,
+                extend_categories=True,
+                **registry[_SETUP_ARGS_KEY]
+            )
+
+        model = _initialize_model(cls, adata, attr_dict)
+        adata_manager = model.get_anndata_manager(adata, required=True)
+
+        version_split = adata_manager.registry[_constants._SCVI_VERSION_KEY].split(".")
         if version_split[1] < "8" and version_split[0] == "0":
             warnings.warn(
                 "Query integration should be performed using models trained with version >= 0.8"
             )
 
-        transfer_anndata_setup(scvi_setup_dict, adata, extend_categories=True)
-
-        model = _initialize_model(cls, adata, attr_dict)
-
-        # set saved attrs for loaded model
-        for attr, val in attr_dict.items():
-            setattr(model, attr, val)
-
         model.to_device(device)
 
         # model tweaking
