@@ -8,7 +8,7 @@ import torch.nn.functional as F
 from torch.distributions import Normal
 from torch.distributions import kl_divergence as kl
 
-from scvi import _CONSTANTS
+from scvi import REGISTRY_KEYS
 from scvi._compat import Literal
 from scvi.distributions import (
     NegativeBinomial,
@@ -82,8 +82,17 @@ class TOTALVAE(BaseModuleClass):
         Array of proteins by batches, the prior initialization for the protein background mean (log scale)
     protein_background_prior_scale
         Array of proteins by batches, the prior initialization for the protein background scale (log scale)
+    use_size_factor_key
+        Use size_factor AnnDataField defined by the user as scaling factor in mean of conditional distribution.
+        Takes priority over `use_observed_lib_size`.
     use_observed_lib_size
         Use observed library size for RNA as scaling factor in mean of conditional distribution
+    library_log_means
+        1 x n_batch array of means of the log library sizes. Parameterizes prior on library size if
+        not using observed library size.
+    library_log_vars
+        1 x n_batch array of variances of the log library sizes. Parameterizes prior on library size if
+        not using observed library size.
     """
 
     def __init__(
@@ -109,7 +118,10 @@ class TOTALVAE(BaseModuleClass):
         encode_covariates: bool = True,
         protein_background_prior_mean: Optional[np.ndarray] = None,
         protein_background_prior_scale: Optional[np.ndarray] = None,
+        use_size_factor_key: bool = False,
         use_observed_lib_size: bool = True,
+        library_log_means: Optional[np.ndarray] = None,
+        library_log_vars: Optional[np.ndarray] = None,
         use_batch_norm: Literal["encoder", "decoder", "none", "both"] = "both",
         use_layer_norm: Literal["encoder", "decoder", "none", "both"] = "none",
     ):
@@ -125,8 +137,22 @@ class TOTALVAE(BaseModuleClass):
         self.protein_dispersion = protein_dispersion
         self.latent_distribution = latent_distribution
         self.protein_batch_mask = protein_batch_mask
-        self.use_observed_lib_size = use_observed_lib_size
         self.encode_covariates = encode_covariates
+        self.use_size_factor_key = use_size_factor_key
+        self.use_observed_lib_size = use_size_factor_key or use_observed_lib_size
+        if not self.use_observed_lib_size:
+            if library_log_means is None or library_log_means is None:
+                raise ValueError(
+                    "If not using observed_lib_size, "
+                    "must provide library_log_means and library_log_vars."
+                )
+
+            self.register_buffer(
+                "library_log_means", torch.from_numpy(library_log_means).float()
+            )
+            self.register_buffer(
+                "library_log_vars", torch.from_numpy(library_log_vars).float()
+            )
 
         # parameters for prior on rate_back (background protein mean)
         if protein_background_prior_mean is None:
@@ -212,6 +238,7 @@ class TOTALVAE(BaseModuleClass):
             dropout_rate=dropout_rate_decoder,
             use_batch_norm=use_batch_norm_decoder,
             use_layer_norm=use_layer_norm_decoder,
+            scale_activation="softplus" if use_size_factor_key else "softmax",
         )
 
     def get_sample_dispersion(
@@ -297,14 +324,14 @@ class TOTALVAE(BaseModuleClass):
         return reconst_loss_gene, reconst_loss_protein
 
     def _get_inference_input(self, tensors):
-        x = tensors[_CONSTANTS.X_KEY]
-        y = tensors[_CONSTANTS.PROTEIN_EXP_KEY]
-        batch_index = tensors[_CONSTANTS.BATCH_KEY]
+        x = tensors[REGISTRY_KEYS.X_KEY]
+        y = tensors[REGISTRY_KEYS.PROTEIN_EXP_KEY]
+        batch_index = tensors[REGISTRY_KEYS.BATCH_KEY]
 
-        cont_key = _CONSTANTS.CONT_COVS_KEY
+        cont_key = REGISTRY_KEYS.CONT_COVS_KEY
         cont_covs = tensors[cont_key] if cont_key in tensors.keys() else None
 
-        cat_key = _CONSTANTS.CAT_COVS_KEY
+        cat_key = REGISTRY_KEYS.CAT_COVS_KEY
         cat_covs = tensors[cat_key] if cat_key in tensors.keys() else None
 
         input_dict = dict(
@@ -315,15 +342,20 @@ class TOTALVAE(BaseModuleClass):
     def _get_generative_input(self, tensors, inference_outputs):
         z = inference_outputs["z"]
         library_gene = inference_outputs["library_gene"]
-        batch_index = tensors[_CONSTANTS.BATCH_KEY]
-        label = tensors[_CONSTANTS.LABELS_KEY]
+        batch_index = tensors[REGISTRY_KEYS.BATCH_KEY]
+        label = tensors[REGISTRY_KEYS.LABELS_KEY]
 
-        cont_key = _CONSTANTS.CONT_COVS_KEY
+        cont_key = REGISTRY_KEYS.CONT_COVS_KEY
         cont_covs = tensors[cont_key] if cont_key in tensors.keys() else None
 
-        cat_key = _CONSTANTS.CAT_COVS_KEY
+        cat_key = REGISTRY_KEYS.CAT_COVS_KEY
         cat_covs = tensors[cat_key] if cat_key in tensors.keys() else None
 
+        size_factor_key = REGISTRY_KEYS.SIZE_FACTOR_KEY
+        size_factor = (
+            tensors[size_factor_key] if size_factor_key in tensors.keys() else None
+        )
+
         return dict(
             z=z,
             library_gene=library_gene,
@@ -331,6 +363,7 @@ class TOTALVAE(BaseModuleClass):
             label=label,
             cat_covs=cat_covs,
             cont_covs=cont_covs,
+            size_factor=size_factor,
         )
 
     @auto_move_data
@@ -342,6 +375,7 @@ class TOTALVAE(BaseModuleClass):
         label: torch.Tensor,
         cont_covs=None,
         cat_covs=None,
+        size_factor=None,
         transform_batch: Optional[int] = None,
     ) -> Dict[str, Union[torch.Tensor, Dict[str, torch.Tensor]]]:
         decoder_input = z if cont_covs is None else torch.cat([z, cont_covs], dim=-1)
@@ -353,8 +387,11 @@ class TOTALVAE(BaseModuleClass):
         if transform_batch is not None:
             batch_index = torch.ones_like(batch_index) * transform_batch
 
+        if not self.use_size_factor_key:
+            size_factor = library_gene
+
         px_, py_, log_pro_back_mean = self.decoder(
-            decoder_input, library_gene, batch_index, *categorical_input
+            decoder_input, size_factor, batch_index, *categorical_input
         )
 
         if self.gene_dispersion == "gene-label":
@@ -528,12 +565,6 @@ class TOTALVAE(BaseModuleClass):
             tensor of values with shape ``(batch_size, n_input_genes)``
         y
             tensor of values with shape ``(batch_size, n_input_proteins)``
-        local_l_mean_gene
-            tensor of means of the prior distribution of latent variable l
-            with shape ``(batch_size, 1)````
-        local_l_var_gene
-            tensor of variancess of the prior distribution of latent variable l
-            with shape ``(batch_size, 1)``
         batch_index
             array that indicates which batch the cells belong to with shape ``batch_size``
         label
@@ -551,11 +582,9 @@ class TOTALVAE(BaseModuleClass):
         px_ = generative_outputs["px_"]
         py_ = generative_outputs["py_"]
 
-        x = tensors[_CONSTANTS.X_KEY]
-        batch_index = tensors[_CONSTANTS.BATCH_KEY]
-        local_l_mean_gene = tensors[_CONSTANTS.LOCAL_L_MEAN_KEY]
-        local_l_var_gene = tensors[_CONSTANTS.LOCAL_L_VAR_KEY]
-        y = tensors[_CONSTANTS.PROTEIN_EXP_KEY]
+        x = tensors[REGISTRY_KEYS.X_KEY]
+        batch_index = tensors[REGISTRY_KEYS.BATCH_KEY]
+        y = tensors[REGISTRY_KEYS.PROTEIN_EXP_KEY]
 
         if self.protein_batch_mask is not None:
             pro_batch_mask_minibatch = torch.zeros_like(y)
@@ -575,9 +604,16 @@ class TOTALVAE(BaseModuleClass):
         # KL Divergence
         kl_div_z = kl(Normal(qz_m, torch.sqrt(qz_v)), Normal(0, 1)).sum(dim=1)
         if not self.use_observed_lib_size:
+            n_batch = self.library_log_means.shape[1]
+            local_library_log_means = F.linear(
+                one_hot(batch_index, n_batch), self.library_log_means
+            )
+            local_library_log_vars = F.linear(
+                one_hot(batch_index, n_batch), self.library_log_vars
+            )
             kl_div_l_gene = kl(
                 Normal(ql_m, torch.sqrt(ql_v)),
-                Normal(local_l_mean_gene, torch.sqrt(local_l_var_gene)),
+                Normal(local_library_log_means, torch.sqrt(local_library_log_vars)),
             ).sum(dim=1)
         else:
             kl_div_l_gene = 0.0
@@ -586,9 +622,11 @@ class TOTALVAE(BaseModuleClass):
             Normal(py_["back_alpha"], py_["back_beta"]), self.back_mean_prior
         )
         if pro_batch_mask_minibatch is not None:
-            kl_div_back_pro = (pro_batch_mask_minibatch * kl_div_back_pro_full).sum(
-                dim=1
+            kl_div_back_pro = torch.zeros_like(kl_div_back_pro_full)
+            kl_div_back_pro.masked_scatter_(
+                pro_batch_mask_minibatch.bool(), kl_div_back_pro_full
             )
+            kl_div_back_pro = kl_div_back_pro.sum(dim=1)
         else:
             kl_div_back_pro = kl_div_back_pro_full.sum(dim=1)
         loss = torch.mean(
@@ -609,7 +647,7 @@ class TOTALVAE(BaseModuleClass):
             kl_div_back_pro=kl_div_back_pro,
         )
 
-        return LossRecorder(loss, reconst_losses, kl_local, kl_global=0.0)
+        return LossRecorder(loss, reconst_losses, kl_local, kl_global=torch.tensor(0.0))
 
     @torch.no_grad()
     def sample(self, tensors, n_samples=1):
@@ -639,9 +677,8 @@ class TOTALVAE(BaseModuleClass):
     @torch.no_grad()
     @auto_move_data
     def marginal_ll(self, tensors, n_mc_samples):
-        x = tensors[_CONSTANTS.X_KEY]
-        local_l_mean = tensors[_CONSTANTS.LOCAL_L_MEAN_KEY]
-        local_l_var = tensors[_CONSTANTS.LOCAL_L_VAR_KEY]
+        x = tensors[REGISTRY_KEYS.X_KEY]
+        batch_index = tensors[REGISTRY_KEYS.BATCH_KEY]
         to_sum = torch.zeros(x.size()[0], n_mc_samples)
 
         for i in range(n_mc_samples):
@@ -664,24 +701,37 @@ class TOTALVAE(BaseModuleClass):
             reconst_loss_protein = reconst_loss["reconst_loss_protein"]
 
             # Log-probabilities
-            p_l_gene = (
-                Normal(local_l_mean, local_l_var.sqrt())
-                .log_prob(log_library)
-                .sum(dim=-1)
-            )
+            log_prob_sum = torch.zeros(qz_m.shape[0]).to(self.device)
+
+            if not self.use_observed_lib_size:
+                n_batch = self.library_log_means.shape[1]
+                local_library_log_means = F.linear(
+                    one_hot(batch_index, n_batch), self.library_log_means
+                )
+                local_library_log_vars = F.linear(
+                    one_hot(batch_index, n_batch), self.library_log_vars
+                )
+                p_l_gene = (
+                    Normal(local_library_log_means, local_library_log_vars.sqrt())
+                    .log_prob(log_library)
+                    .sum(dim=-1)
+                )
+                q_l_x = Normal(ql_m, ql_v.sqrt()).log_prob(log_library).sum(dim=-1)
+
+                log_prob_sum += p_l_gene - q_l_x
+
             p_z = Normal(0, 1).log_prob(z).sum(dim=-1)
             p_mu_back = self.back_mean_prior.log_prob(log_pro_back_mean).sum(dim=-1)
             p_xy_zl = -(reconst_loss_gene + reconst_loss_protein)
             q_z_x = Normal(qz_m, qz_v.sqrt()).log_prob(z).sum(dim=-1)
-            q_l_x = Normal(ql_m, ql_v.sqrt()).log_prob(log_library).sum(dim=-1)
             q_mu_back = (
                 Normal(py_["back_alpha"], py_["back_beta"])
                 .log_prob(log_pro_back_mean)
                 .sum(dim=-1)
             )
-            to_sum[:, i] = (
-                p_z + p_l_gene + p_mu_back + p_xy_zl - q_z_x - q_l_x - q_mu_back
-            )
+            log_prob_sum += p_z + p_mu_back + p_xy_zl - q_z_x - q_mu_back
+
+            to_sum[:, i] = log_prob_sum
 
         batch_log_lkl = torch.logsumexp(to_sum, dim=-1) - np.log(n_mc_samples)
         log_lkl = torch.sum(batch_log_lkl).item()
